import numpy as np
import open3d as o3d
import CloudFunc
from scipy.spatial.transform import Rotation as R

################################点云降采样#################################

def voxel_down_sample(pcd, voxel_size):
    """
    体素方法点云降采样
    :param pcd: 点云数据
    :param voxel_size: 体素降采样的网格长度
    :return: 降采样点云数据
    """
    return pcd.voxel_down_sample(voxel_size=voxel_size)

def add_rgb_to_pcd(pcd):
    """
    给点云天啊及颜色信息
    :param pcd: 点云数据
    :return:
    """
    # 为点云中的每个点指定RGB颜色，这里我们使用红色作为示例
    # RGB值范围为[0, 1]，红色可以表示为(1, 0, 0)
    pcd = np.asarray(pcd.points).shape[0]
    colors = np.repeat([[1, 0, 0]], pcd, axis=0)  # 创建一个全是红色的颜色数组
    # 将颜色信息添加到点云对象中
    pcd.colors = o3d.utility.Vector3dVector(colors)
    return pcd

print("读取txt点云并展示")
file_path = '/home/lzx/catkin_ws_LstRobot/src/publish_pointcloud/data/Ts.txt'
pcd = CloudFunc.open_file(file_path)

# print("用长度为n的体素网格进行降采样，长度越长，降采样能力越强,直接用10和从1逐步到10的降采样结果会更加杂乱一点更贴近真实的感觉")
pcd = pcd.voxel_down_sample(voxel_size=2)
print('保存降采样信息')
CloudFunc.save_file(pcd,'/home/lzx/catkin_ws_LstRobot/src/publish_pointcloud/data/Ts_down.pcd')

################################去掉焊缝周围的点云#################################

# 焊缝起点终点
# A = np.array([976.2632995614695, -146.89913794322, 296.23875340524336])
# B = np.array([1448.1832002237788, -126.02872682427278, 293.0279482693677])
A = np.array([1408.9291859593534, -463.1764424856121, 644.0476051625956])
B = np.array([1421.5816519558423, -459.93745136356836, 387.3775419764174])

u = B - A  # 焊缝向量
c = np.linalg.norm(u)

# 使用pcread直接读取点云数据
filename = '/home/lzx/catkin_ws_LstRobot/src/publish_pointcloud/data/Ts_down.pcd'
ptCloud = o3d.io.read_point_cloud(filename)
data = np.asarray(ptCloud.points)

# 初始化一个新的空数组来存储保留的点云数据
data_retained = []

for i in range(data.shape[0]):
    P = data[i]
    v = P - A
    distance = np.linalg.norm(np.cross(u, v)) / c
    if distance >= 5:
        # 将满足条件的点加入新的数组中
        data_retained.append(P)

################################计算焊枪位姿#################################













# 将点云数据的小数点向前移动三位（即乘以1000）
data_scaled = np.array(data_retained) / 1000

# 创建一个新的pointCloud对象来存储缩放后的数据
ptCloud_scaled = o3d.geometry.PointCloud()
ptCloud_scaled.points = o3d.utility.Vector3dVector(data_scaled)

# 定义新的PCD文件名
new_filename = '/home/lzx/catkin_ws_LstRobot/src/publish_pointcloud/data/Ts_octomap.pcd'

# 将缩放后的点云数据写入新的 PCD 文件
o3d.io.write_point_cloud(new_filename, ptCloud_scaled, write_ascii=True)

print('Scaled point cloud saved to', new_filename)

# 读取保存的点云数据
pcd = o3d.io.read_point_cloud(new_filename)

# 显示点云
o3d.visualization.draw_geometries([pcd])